function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import { useState, useCallback, useEffect, useRef } from "react";

const useSocketRegistry = (delay = 100) => {
  const sockets = useRef({});
  const [counts, setCounts] = useState({});
  const acquire = useCallback((url, keepAlive, cb) => {
    const t = setTimeout(() => {
      let socket = sockets.current[url];

      if (!socket || socket && keepAlive && socket.readyState != WebSocket.CONNECTING && socket.readyState != WebSocket.OPEN) {
        socket = new WebSocket(url);
        sockets.current[url] = socket;
      }

      setCounts(cnts => _objectSpread({}, cnts, {
        [url]: (cnts[url] || 0) + 1
      }));
      cb(socket);
    }, delay);
    return () => clearTimeout(t);
  }, [delay]);
  const release = useCallback(url => {
    setCounts(counts => {
      const count = counts[url];
      if (!count) return counts;
      const newCount = count - 1;
      return _objectSpread({}, counts, {
        [url]: newCount
      });
    });
  }, []);
  useEffect(() => {
    // cleanup zero-ref sockets
    const t = setTimeout(() => {
      for (let url in counts) {
        if (counts[url] == 0 && sockets.current[url]) {
          sockets.current[url].close();
          delete sockets.current[url];
        }
      }
    }, delay + 50);
    return () => clearTimeout(t);
  }, [counts, delay]);
  return {
    acquire,
    release
  };
};

export default useSocketRegistry;