"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = require("react");

var _scope = require("./scope");

var _registry = _interopRequireDefault(require("./registry"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const useSocketAcquisition = socketDelay => {
  const {
    acquire,
    release
  } = (0, _registry.default)(socketDelay);
  const acquireScoped = (0, _scope.useAcquireSocket)();
  const releaseScoped = (0, _scope.useReleaseSocket)();
  return {
    acquire: acquireScoped || acquire,
    release: releaseScoped || release
  };
};

const useSocketInstance = (url, keepAlive, keepAliveSignal, socketDelay) => {
  const [socket, setSocket] = (0, _react.useState)(null);
  const {
    acquire,
    release
  } = useSocketAcquisition(socketDelay);
  (0, _react.useEffect)(() => {
    let sock, cancelAcquire;

    if (url) {
      cancelAcquire = acquire(url, keepAlive, s => {
        sock = s;
        setSocket(s);
        cancelAcquire = null;
      });
    }

    return () => {
      if (cancelAcquire) {
        cancelAcquire();
      }

      if (sock) {
        release(sock.url);
      }
    };
  }, [url, keepAlive, keepAliveSignal, acquire, release]);
  return socket;
};

var _default = useSocketInstance;
exports.default = _default;